@@ -20,6 +20,8 @@ from canalyst_candas.utils import (
     read_json,
     save_guidance_csv,
     send_scenario,
+    calendar_quarter,
+    get_excel_model,
 )
 import __main__
 from pyvis.network import Network
@@ -91,6 +93,135 @@ class FuncMusic(object):
         return float(self.value) - float(self.argument)
 
 
+class Search:
+    def __init__(self, config: Config = None):
+        self.config = config
+        self.df_search = None
+        self.df_guidance = None
+
+    def help(self):
+        print("search_time_series syntax:")
+        print(
+            "ticker (Bloomberg ticker),sector,time_series_name,time_series_description,category,is_driver,unit_type"
+        )
+        print("unit types:currency, percentage, count, ratio, time")
+        print("search_guidance_time_series syntax:")
+        print(
+            "ticker (Bloomberg ticker),sector,item (time series description),item_name (time series name),most_recent (True or blank)"
+        )
+        return
+
+    def search_guidance_time_series(
+        self,
+        ticker="",
+        sector="",
+        Type="",
+        item="",
+        item_name="",
+        most_recent="",
+    ):
+        if self.df_guidance is None:
+            print("Building guidance index...")
+            self.df_guidance = Getter(config=self.config).get_csv_from_s3(
+                f"DATA/df_guidance.csv"
+            )
+            print("Done")
+
+        df = self.df_guidance
+
+        if most_recent == True:
+            df = df.sort_values(["ticker", "Date"], ascending=False)
+            df = df.groupby(["ticker", "Item"]).first().reset_index()
+
+        if type(ticker) == list:
+            df = df.loc[df["ticker"].isin(ticker)]
+        elif ticker != "":
+            df = df.loc[df["ticker"].str.contains(ticker, case=True, regex=True)]
+
+        if sector != "":
+            df = df.loc[df["Path"].str.contains(sector, case=True, regex=True)]
+        if item != "":
+            df = df.loc[df["Item"].str.contains(item, case=False, regex=True)]
+        if item_name != "":
+            df = df.loc[df["item_name"].str.contains(item_name, case=False)]
+        return df[
+            [
+                "ticker",
+                "Path",
+                "Item",
+                "Item Name",
+                "Fiscal Period",
+                "Low",
+                "Mid",
+                "High",
+                "Type.1",
+                "Date",
+                "Link",
+            ]
+        ].sort_values(["ticker", "Date", "Item"])
+
+    def search_time_series(
+        self,
+        ticker="",
+        sector="",
+        time_series_name="",
+        time_series_description="",
+        category="",
+        is_driver="",
+        unit_type="",
+    ):
+        if self.df_search is None:
+            print("Building search index...")
+            self.df_search = Getter(config=self.config).get_csv_from_s3(
+                f"DATA/df_search.csv"
+            )
+            print("Done")
+
+        df = self.df_search
+        if type(ticker) == list:
+            df = df.loc[df["ticker"].isin(ticker)]
+        elif ticker != "":
+            df = df.loc[df["ticker"].str.contains(ticker, case=True, regex=True)]
+
+        if sector != "":
+            df = df.loc[df["Path"].str.contains(sector, case=True, regex=True)]
+        if category != "":
+            df = df.loc[df["category"].str.contains(category, case=True, regex=True)]
+        if time_series_name != "":
+            df = df.loc[
+                df["time_series_name"].str.contains(
+                    time_series_name, case=False, regex=True
+                )
+            ]
+        if time_series_description != "":
+            df = df.loc[
+                df["time_series_description"].str.contains(
+                    time_series_description, case=False
+                )
+            ]
+        if is_driver != "":
+            df = df.loc[df["is_driver"] == is_driver]
+        if unit_type != "":
+            df = df.loc[
+                df["unit_type"] == unit_type
+            ]  # currency, percentage, count, ratio, time
+        df["fiscal_or_annual"] = np.where(
+            df["value"].isna(), "fiscal_year", "fiscal_quarter"
+        )
+        return df[
+            [
+                "ticker",
+                "Path",
+                "category",
+                "time_series_description",
+                "time_series_name",
+                "fiscal_or_annual",
+                "unit_type",
+                "is_driver",
+            ]
+        ].sort_values(["ticker", "time_series_description"])
+
+
 # WIP Class for parsing json revenue builds in graph structure (as opposed to tree)
 # class FormulaGraph:
 # def __init__(
@@ -566,6 +697,25 @@ class ModelSet:
         if type(ticker_list) is str:
             ticker_list = [ticker_list]
 
+    def help(self, function_name=""):
+        dict_help = {
+            "create_model_map": "Create a model map.  params: ticker, col_for_labels = 'time_series_name', time_series_name = 'MO_RIS_REV', tree = True, notebook = True",
+            "create_time_series_chart": "Create a time series chart.  params: ticker, time_series_name",
+            "guidance": "Return guidance from a mmodel.  params: ticker",
+            "mrq": "Return most recent quarter from a mmodel.  params: ticker",
+            "time_series_search": "Return time series regex match. params: time_series_name",
+            "driver_search": "Return driver regex match. params: driver_name",
+            "model_frame": "Return a dataframe of the full modelset. params: time_series_name,period_name,is_driver='',pivot=False,mrq=False,period_duration_type='',is_historical='',n_periods='',mrq_notation=False",
+            "forecast_frame": "Return a params dataframe for use in the fit function. params: time_series_name, n_periods, function_name='value', function_value='' where function name can be add, subtract, multiply, divide, or value",
+            "fit": "Return a return series for a fitted model. params: params dataframe, return_series.  params dataframe columns are: ticker period time_series_name value new_value",
+        }
+        if function_name != "":
+            return dict_help[function_name]
+        else:
+            df = pd.DataFrame(dict_help, index=[0]).T
+            df.columns = ["help"]
+            return df
+
     # shows a tree for the given ticker and time series
     def create_model_map(
         self,
@@ -575,17 +725,68 @@ class ModelSet:
         tree=True,
         notebook=True,
     ):
-        model = self.models[ticker]
-        return model.create_model_map(
-            time_series_name=time_series_name,
-            tree=tree,
-            col_for_labels=col_for_labels,
-            notebook=notebook,
-            common_time_series=self.common_time_series,
-        )
+        if type(ticker) == list:
+            print("Please request one ticker at a time.")
+            return
+
+        if ticker in self.ticker_list:
+            model = self.models[ticker]
+            return model.create_model_map(
+                time_series_name=time_series_name,
+                tree=tree,
+                col_for_labels=col_for_labels,
+                notebook=notebook,
+                common_time_series_names=self._common_time_series_names,
+            )
+        else:
+            print("Please choose a ticker in this ModelSet's ticker list")
+            return
+
+    def create_time_series_chart(self, ticker, time_series_name):
+        if type(ticker) == list:
+            print("Please request one ticker at a time.")
+            return
+
+        if ticker in self.ticker_list:
+            self.models[df].create_time_series_chart(time_series_name)
+
+        else:
+            print("Please choose a ticker in this ModelSet's ticker list")
+        return
+
+    def guidance(self, ticker):
+        if type(ticker) == list:
+            list_df = []
+            for t in ticker:
+                if ticker in self.ticker_list:
+                    df = self.models[ticker].guidance()
+                    list_df.append(df)
+            return pd.concat(list_df)
+        else:
+            if ticker in self.ticker_list:
+                return self.models[ticker].guidance()
+            else:
+                print("Please choose a ticker in this ModelSet's ticker list")
+        return
+
+    def mrq(self, ticker):
+        if type(ticker) == list:
+            list_df = []
+            for t in ticker:
+                if ticker in self.ticker_list:
+                    df = self.models[ticker].mrq()
+                    df["ticker"] = t
+                    list_df.append(df)
+            return pd.concat(list_df)
+        else:
+            if ticker in self.ticker_list:
+                return self.models[ticker].mrq()
+            else:
+                print("Please choose a ticker in this ModelSet's ticker list")
+        return
 
     # get a list of time series from the default dataframe
-    def get_featurelist(self):
+    def time_series_names(self):
         df = self._features
         df = (
             df.groupby(["time_series_name"])
@@ -696,7 +897,7 @@ class ModelSet:
         return
 
     # allows for filtering and shaping the default dataframe
-    # MODEL model_frame
+    # MODELSET model_frame
     def model_frame(
         self,
         time_series_name="",
@@ -709,10 +910,16 @@ class ModelSet:
         n_periods="",
         mrq_notation=False,
         warning=True,
+        ticker="",
     ):
 
+        if ticker != "":
+            df = self.models[ticker]._model_frame
+        else:
+            df = self._features
+
         return filter_dataset(
-            self._features,  # this is the core dataset of the modelset class
+            df,  # this is the core dataset of the modelset class
             time_series_name,
             period_name,
             is_driver,
@@ -936,7 +1143,12 @@ class ModelMap:
 
         # Create dataframe for precedent/dependent tree, graph of the tree, and a network of that graph (for visualization)
         self.df = self.load_data()
-        self.network = self.create_model()
+        try:
+            self.network = self.create_model()
+        except:
+            print(
+                "Canalyst-Candas error creating modelmap.  Possible permissions issue with your Canalyst login, please contact us."
+            )
 
         self.fig = None
 
@@ -1190,6 +1402,15 @@ class ModelMap:
                     list_out.append(node["title"].split(":")[0])
         return list_out
 
+    # lists out all the nodes - duplicate of above for backwards comp
+    def time_series_names(self, search=""):
+        list_out = []
+        if self.nodes:
+            for node in self.nodes:
+                if re.match(f".*{search}.*", node["title"]):
+                    list_out.append(node["title"].split(":")[0])
+        return list_out
+
     # creates chart of given time series:
     def create_time_series_chart(self, time_series_name):
         # get needed data from dataframe
@@ -1243,9 +1464,12 @@ class Model:
         if self.force == True:
             print("model_frame")
 
-        self.csin, self.latest_version = get_company_info_from_ticker(
-            self.ticker, self.api_headers, self.config.mds_host
-        )
+        try:
+            self.csin, self.latest_version = get_company_info_from_ticker(
+                self.ticker, self.api_headers, self.config.mds_host
+            )
+        except:
+            print("Canalyst-Candas: Error on API Key.  Perhaps missing key.")
 
         self.get_model_frame()
 
@@ -1271,6 +1495,56 @@ class Model:
             print("guidance")
             self.create_guidance_csv()
 
+        self._model_name = self.get_excel_model_name()
+        self._model_frame["model_name"] = self._model_name
+
+    def get_excel_model_name(self):
+
+        csin = self.csin
+        auth_headers = self.api_headers
+        mds_host = self.config.mds_host
+        wp_host = self.config.wp_host
+
+        from python_graphql_client import GraphqlClient
+
+        client = GraphqlClient(endpoint=f"{wp_host}/model-workbooks")
+
+        # Create the query string and variables required for the request.
+        query = """
+        query driversWorksheetByCSIN($csin: ID!) {
+            modelSeries(id: $csin) {
+            latestModel {
+                id
+                name
+                publishedAt
+                variantsByDimensions(
+                    driversWorksheets: [STANDARD_FCF],
+                    periodOrder: [CHRONOLOGICAL],
+                ) {
+                id
+                downloadUrl
+                variantDimensions {
+                    driversWorksheets
+                    periodOrder
+                }
+                }
+            }
+            }
+        }
+        """
+        variables = {"csin": csin}
+
+        # Synchronous request
+        data = client.execute(
+            query=query, variables=variables, headers=auth_headers, verify=False
+        )
+        url = data["data"]["modelSeries"]["latestModel"]["variantsByDimensions"][0][
+            "downloadUrl"
+        ]
+        file_ticker = self.ticker.split(" ")[0]
+        file_name = data["data"]["modelSeries"]["latestModel"]["name"]
+        return file_name
+
     def key_driver_map(self, time_series_name):
         # defaulting to MO_RIS_REV, but you could choose another time_series_name as a starting point like MO_RIS_EBIT
         model_map = self.create_model_map(
@@ -1366,7 +1640,7 @@ class Model:
         col_for_labels="time_series_name",
         tree=True,
         notebook=False,
-        common_time_series=None,
+        common_time_series_names=None,
     ):
         mm = ModelMap(
             config=self.config,
@@ -1375,7 +1649,7 @@ class Model:
             col_for_labels=col_for_labels,
             tree=tree,
             notebook=notebook,
-            common_time_series=common_time_series,
+            common_time_series=common_time_series_names,
         )
         self.model_map = mm
         return mm
@@ -1414,7 +1688,12 @@ class Model:
         data = {"changes": list_changes, "name": scenario_name}
 
         response = send_scenario(
-            ticker=self.ticker, data=data, mds_host=self.config.mds_host
+            self.ticker,
+            data,
+            self.api_headers,
+            self.csin,
+            self.latest_version,
+            self.config.mds_host,
         )
 
         return
@@ -1432,7 +1711,7 @@ class Model:
             time_series=time_series,
             tree=tree,
             notebook=notebook,
-            common_time_series=common_time_series,
+            common_time_series=self._common_time_series_names,
         )
         self.model_map = mm
         return mm.show()
