@@ -429,10 +429,11 @@ def get_drivers_from_model(
             except:
                 log.write(f"error: {defined_name}")
                 continue
-            i_toggle = 0
+
             dict_values = {}
 
             for index, item in enumerate(list_values):
+                i_toggle = 0
                 if type(item.value) == float or type(item.value) == int:
                     dict_values = {}
                     num = (
@@ -1073,6 +1074,7 @@ def mrq_df(df):
         values="value",
         index=[
             "name_index",
+            "model_name",
             "ticker",
             "category",
             "time_series_name",
@@ -1087,6 +1089,7 @@ def mrq_df(df):
 
     name_cols = [
         "name_index",
+        "model_name",
         "ticker",
         "category",
         "time_series_name",
@@ -1110,22 +1113,11 @@ def pivot_df(df, mrq_notation):
         df = mrq_df(df)
         return df
 
-    cols1 = [
-        "ticker",
-        "name_index",
-        "category",
-        "time_series_name",
-        "time_series_description",
-        "is_driver",
-    ]
-    cols1.extend(list(df["period_name"]))
-    cols2 = []
-    [cols2.append(x) for x in cols1 if x not in cols2]
-
     df = pd.pivot_table(
         df,
         values="value",
         index=[
+            "model_name",
             "ticker",
             "name_index",
             "category",
